-- Testing
dofile("noisemoduletree.lua")

testisland=
{
	-- The main shape of the continent is founded on a fuzzy disk
	{name="ContinentDisk", type="greyscale_fuzzydisk", radius=0.75, centerx=0.5, centery=0.5, seamless=false},
	-- Add some X and Y axis turbulence to the fuzzy disk
	{name="ContinentXTurb", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		frequency=3, num_octaves=10, remaprange=true, low=-1, high=1},
	{name="ContinentYTurb", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		frequency=3, num_octaves=10, remaprange=true, low=-1, high=1},
	{name="ContinentTurb", type="greyscale_turbulence", main_source="ContinentDisk", xaxis_source="ContinentXTurb", yaxis_source="ContinentYTurb",
		xpower=100, ypower=100, seamless=true},
	-- Remap the fuzzy disk function to the range [0,1]
	{name="ContinentRemap", type="greyscale_remaprange", source="ContinentTurb", low=0, high=1},
	
	-- Create a selection mask to outline the shape of the continent and blend between ocean floor and continent landforms.
	{name="GConstant0", type="greyscale_constant", constant=0},
	{name="GConstant1", type="greyscale_constant", constant=1},
	{name="ContinentMask", type="greyscale_select", main_source="ContinentRemap", low_source="GConstant0", high_source="GConstant1",
		threshold=0.3, falloff=0.15},
		
	-- Create a selection mask to define the shoreline
	{name="ShorelineMask", type="greyscale_select", main_source="ContinentRemap", low_source="GConstant0", high_source="GConstant1",
		threshold=0.15, falloff=0.01},
	
		
	-- Create a fractal for the ocean floor
	{name="OceanFloorFractal", type="greyscale_fractal", fractal_type="HYBRIDMULTI", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=6},
	{name="OceanFloorRemap", type="greyscale_remaprange", source="OceanFloorFractal", low=0, high=0.3},
	
	
	-- Create a fractal for mountains
	{name="MountainFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=10, frequency=22, remaprange=true, low=0.45,high=1},
		
	-- Create a fractal for rolling hills
	{name="RollingHillsFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=6, remaprange=true, low=0.25, high=0.45},
		
	-- Create a fractal mask to separate areas of mountain from areas of non-mountain
	{name="MountainMaskFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=6, remaprange=true, low=0, high=1},
	{name="MountainMask", type="greyscale_select", main_source="MountainMaskFractal", low_source="GConstant0", high_source="GConstant1",
		threshold=0.7, falloff=0.19},
		
	-- Create a blender to combine mountains with lower terrain
	{name="MountainLowlandBlend", type="greyscale_blend", main_source="MountainMask", low_source="RollingHillsFractal", high_source="MountainFractal"},
	
	
	{name="ContinentBaseAdder", type="greyscale_combiner", combiner_type=0, source_0="ContinentBase", source_1="MountainLowlandBlend"},
	
	-- Continent blend
	{name="ContinentBlend", type="greyscale_blend", main_source="ContinentMask", low_source="OceanFloorRemap", high_source="MountainLowlandBlend"},
		
		
	{name="ContinentBump", type="greyscale_bumpmap", source="ContinentBlend", sharpness=4, light={1.5,3.5,-1.5}},
	
	
	
	-- RGBA Modules
	{name="RGBAConstantBlueDark", type="rgba_constant", red=0.2, green=0.5, blue=0.9},
	{name="RGBAConstantBlueLight", type="rgba_constant", red=0.5, green=0.6, blue=1},
	{name="RGBAConstantGrey", type="rgba_constant", red=0.5, green=0.5, blue=0.5},
	{name="RGBAConstantWhite", type="rgba_constant", red=1,green=1,blue=1},
	{name="RGBAConstantBrown", type="rgba_constant", red=195/255, green=130/255, blue=46/255},
	{name="RGBAConstantGreen", type="rgba_constant", red=0.2, green=0.7, blue=0.1},
	{name="RGBAConstantDarkGreen", type="rgba_constant", red=0.1, green=0.5, blue=0.2},
	
	-- Create a stretched ridged multi fractal to setup a wave pattern
	{name="WaveRippleFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=3, frequency=12, mapx1=0, mapy1=0, mapx2=1, mapy2=5, remaprange=true, low=0, high=1},
	{name="WaveRippleBump", type="greyscale_bumpmap", source="WaveRippleFractal", sharpness=0.1, light={1.5,3.5,-1.5}},
	{name="WaveLightColor", type="rgba_combinealpha", greyscale_source="WaveRippleFractal", rgba_source="RGBAConstantBlueLight"},
	{name="WaveColor", type="rgba_blend", source1="WaveLightColor", source2="RGBAConstantBlueDark", blendop1=0, blendop2=2},
	
	{name="RGBAWavePattern", type="rgba_multrgbagreyscale", greyscale_source="WaveRippleBump", rgba_source="WaveColor"},
	
	-- Multiply the mountain elevation map by the mountain mask and the shoreline mask in order to get a "real" elevation map for
	-- adding snow to higher elevation mountains,, then set up a selection buffer and do a blend between grey and white
	{name="MountainFractalMultiply", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="MountainFractal", source_1="MountainMask",
		source_2="ContinentMask"},
		
	{name="MountainSnowLineSelect", type="greyscale_select", main_source="MountainFractalMultiply", low_source="GConstant0", high_source="GConstant1",
		threshold=0.65, falloff=0.075},
		
	{name="MountainSnowLayer", type="rgba_combinealpha", greyscale_source="MountainSnowLineSelect", rgba_source="RGBAConstantWhite"},
	{name="MountainSnowBlend", type="rgba_blend", source1="MountainSnowLayer", source2="RGBAConstantGrey", blendop1=0, blendop2=2},
	
	-- Setup a fractal to blend between green and brown for lowland colors
	{name="LowlandBlendMask", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=6, frequency=4, remaprange=true, low=0, high=1},
	{name="LowlandColorLayer", type="rgba_combinealpha", greyscale_source="LowlandBlendMask", rgba_source="RGBAConstantGreen"},
	{name="LowlandColorBlend", type="rgba_blend", source1="LowlandColorLayer", source2="RGBAConstantBrown", blendop1=0, blendop2=2},
	
	-- Setup a tighter version of the mountain mask
	{name="MountainColorMask", type="greyscale_select", main_source="MountainMaskFractal", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0.15},
		
	-- Invert the mountain color mask, set up a forest fractal mask and multiply it by the mountain mask, ie no forest cover in mountains
	{name="MountainColorMaskInvert", type="greyscale_remaprange", source="MountainColorMask", low=1, high=0},
	{name="ForestBlendMask", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=6, frequency=6, remaprange=true, low=0, high=1},
	{name="ForestBlendMaskSelect", type="greyscale_select", main_source="ForestBlendMask", low_source="GConstant0", high_source="GConstant1",
		threshold=0.5, falloff=0.05},
	{name="ForestMask", type="greyscale_combiner", combiner_type=ECT_MULT, source_0="ForestBlendMaskSelect", source_1="MountainColorMaskInvert"},
	
	-- Setup a color blend layer for forest/lowland
	-- First, set up a bump source to make bumpy forest color
	{name="ForestBumpFractal", type="greyscale_fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC",
		num_octaves=8, frequency=64, remaprange=true, low=0, high=1},
	{name="ForestBump", type="greyscale_bumpmap", source="ForestBumpFractal", sharpness=1, light={1.5,3.5,-1.5}},
	{name="ForestColorBump", type="rgba_multrgbagreyscale", greyscale_source="ForestBump", rgba_source="RGBAConstantDarkGreen"},
	
	-- Now, blend in forest color
	{name="ForestColorLayer", type="rgba_combinealpha", greyscale_source="ForestMask", rgba_source="ForestColorBump"},
	{name="VegetationColorBlend", type="rgba_blend", source1="ForestColorLayer", source2="LowlandColorBlend", blendop1=0, blendop2=2},
	
	{name="RGBAMountainLayer", type="rgba_combinealpha", greyscale_source="MountainColorMask", rgba_source="MountainSnowBlend"},
	{name="RGBAMountainBlend", type="rgba_blend", source1="RGBAMountainLayer", source2="VegetationColorBlend", blendop1=0, blendop2=2},
	
	
	-- Blend between land colors and ocean colors using the shoreline mask
	{name="RGBALandMap", type="rgba_combinealpha", greyscale_source="ShorelineMask", rgba_source="RGBAMountainBlend"},
	{name="RGBACombineLandOcean", type="rgba_blend", source1="RGBALandMap", source2="RGBAWavePattern", blendop1=0, blendop2=2},
	
	-- Multiple the whole thing by the elevation bump-map
	{name="RGBACombineBump", type="rgba_multrgbagreyscale", greyscale_source="ContinentBump", rgba_source="RGBACombineLandOcean"},
	
	{name="RGBAAdapter", type="rgba_greyscaleadapter", source="MountainFractalMultiply"},
	{name="Output", type="output", source="RGBACombineBump", mode=0, width=512, height=512}
}

ti=NoiseModuleTree(testisland)
out=ti:getFunction("Output")
ti:setSeedAll(randInt())
out:eval("test.tga")
